// ---------------------------------------------------------------------------
// LTLError.java
// An error class for the LTL model checker.
// $Id: LTLError.java,v 1.1 2022/08/24 12:47:14 schreine Exp $
//
// Author: Ágoston Sütő <agoston.suetoe@jku.at>
// Copyright (C) 2021-, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.jku.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package riscal.ltl;

import riscal.syntax.AST;

/***************************************************************************
 * The exception raised by the LTL checker on an error.
 **************************************************************************/
public final class LTLError extends RuntimeException {
	private static final long serialVersionUID = 14102021L;
	public final AST tree;

	/*************************************************************************
	 * Create an LTL error with the denoted message originating from
	 * processing the denoted phrase.
	 * @param tree the abstract syntax tree of the phrase
	 * @param msg the message
	 ************************************************************************/
	public LTLError(AST tree, String msg) {
		super(msg);
		this.tree = tree;
	}
}
// ----------------------------------------------------------------------------
// end of file
// ----------------------------------------------------------------------------