// ---------------------------------------------------------------------------
// SMT.java
// Settings for the SMT-LIB translation.
// $Id: SMTSettings.java,v 1.3 2020/02/26 07:49:30 schreine Exp $
//
// Authors: Franz-Xaver Reichl <franz.x.reichl@gmail.com>
//      and Wolfgang Schreiner <Wolfgang.Schreiner@risc.jku.at>
// Copyright (C) 2019-, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.jku.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package riscal.smt;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Composite;

import riscal.swt.TopWindow;

public class SMTSettings 
{
  /****************************************************************************
   * Construct window connected to denoted top Window
   * @param top the top window.
   * @return a window connected to the GUI.
   ***************************************************************************/
  public static SMTSettings construct(TopWindow top)
  {
    return new SMTSettings(top);
  }


  /****************************************************************************
   * Create symbol list using denoted top window.
   * @param top the top window.
   * @return a window connected to the GUI.
   ***************************************************************************/
  private SMTSettings(TopWindow top)
  {
    Shell shell = new Shell(top.getDisplay(), SWT.CLOSE | SWT.BORDER);
    shell.setLayout(new GridLayout(1, false));
    shell.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    shell.setText("SMT Configuration"); 

    Group bollectorGr = new Group(shell, SWT.NONE);
    bollectorGr.setText("Boolector");
    bollectorGr.setLayout(new GridLayout(2, false));
    bollectorGr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button boolectorButton = new Button(bollectorGr, SWT.PUSH);
    boolectorButton.setText("Select Executable");

    Text boolectorLocation=new Text(bollectorGr, SWT.LEFT | SWT.BORDER | SWT.READ_ONLY);
    boolectorLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    boolectorLocation.setText(SMT.dirBoolector+File.separator+SMT.boolectorCommandName);

    boolectorButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
        fileDialog.setFilterPath(SMT.dirBoolector);
        fileDialog.setFileName(SMT.boolectorCommandName);
        fileDialog.open();
        String solverName = fileDialog.getFileName();		 
        if (solverName == null) return;
        if (solverName.equals("")) return;
        if (solverName.equals("Untitled")) return;
        String solverDir = fileDialog.getFilterPath();
        if (solverDir == null) return;
        if (solverDir.equals("")) return;
        SMT.dirBoolector=solverDir;
        SMT.boolectorCommandName=solverName;
        boolectorLocation.setText(solverDir+File.separator+solverName);
      }
    });


    Group cvc4Gr = new Group(shell, SWT.NONE);
    cvc4Gr.setText("CVC4");
    cvc4Gr.setLayout(new GridLayout(2, false));
    cvc4Gr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button cvc4Button = new Button(cvc4Gr, SWT.PUSH);
    cvc4Button.setText("Select Executable");
    Text cvc4Location=new Text(cvc4Gr, SWT.LEFT | SWT.BORDER | SWT.READ_ONLY);
    cvc4Location.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    cvc4Location.setText(SMT.dirCVC4+File.separator+SMT.cvc4CommandName);

    cvc4Button.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
        fileDialog.setFilterPath(SMT.dirCVC4);
        fileDialog.setFileName(SMT.cvc4CommandName);
        fileDialog.open();
        String solverName = fileDialog.getFileName();     
        if (solverName == null) return;
        if (solverName.equals("")) return;
        if (solverName.equals("Untitled")) return;
        String solverDir = fileDialog.getFilterPath();
        if (solverDir == null) return;
        if (solverDir.equals("")) return;
        SMT.dirCVC4=solverDir;
        SMT.cvc4CommandName=solverName;
        cvc4Location.setText(solverDir+File.separator+solverName);
      }
    });

    Group yicesGr = new Group(shell, SWT.NONE);
    yicesGr.setText("Yices");
    yicesGr.setLayout(new GridLayout(2, false));
    yicesGr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button yicesButton = new Button(yicesGr, SWT.PUSH);
    yicesButton.setText("Select Executable");
    Text yicesLocation=new Text(yicesGr, SWT.LEFT | SWT.BORDER | SWT.READ_ONLY);
    yicesLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    yicesLocation.setText(SMT.dirYices+File.separator+SMT.yicesCommandName);

    yicesButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
        fileDialog.setFilterPath(SMT.dirYices);
        fileDialog.setFileName(SMT.yicesCommandName);
        fileDialog.open();
        String solverName = fileDialog.getFileName();		 
        if (solverName == null) return;
        if (solverName.equals("")) return;
        if (solverName.equals("Untitled")) return;
        String solverDir = fileDialog.getFilterPath();
        if (solverDir == null) return;
        if (solverDir.equals("")) return;
        SMT.dirYices=solverDir;
        SMT.yicesCommandName=solverName;
        yicesLocation.setText(solverDir+File.separator+solverName);
      }
    });

    Group z3Gr = new Group(shell, SWT.NONE);
    z3Gr.setText("Z3");
    z3Gr.setLayout(new GridLayout(2, false));
    z3Gr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button z3Button = new Button(z3Gr, SWT.PUSH);
    z3Button.setText("Select Executable");
    Text z3Location=new Text(z3Gr, SWT.LEFT | SWT.BORDER | SWT.READ_ONLY);
    z3Location.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    z3Location.setText(SMT.dirZ3+File.separator+SMT.z3CommandName);

    z3Button.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
        fileDialog.setFilterPath(SMT.dirZ3);
        fileDialog.setFileName(SMT.z3CommandName);
        fileDialog.open();
        String solverName = fileDialog.getFileName();     
        if (solverName == null) return;
        if (solverName.equals("")) return;
        if (solverName.equals("Untitled")) return;
        String solverDir = fileDialog.getFilterPath();
        if (solverDir == null) return;
        if (solverDir.equals("")) return;
        SMT.dirZ3=solverDir;
        SMT.z3CommandName=solverName;
        z3Location.setText(solverDir+File.separator+solverName);
      }
    });


    Group scriptGr = new Group(shell, SWT.NONE);
    scriptGr.setText("SMT-LIB Script");
    scriptGr.setLayout(new GridLayout(1, false));
    scriptGr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Composite scriptRow1 = new Composite(scriptGr, SWT.NONE);
    scriptRow1.setLayout(new GridLayout(2, false));
    scriptRow1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button scriptButton = new Button(scriptRow1, SWT.PUSH);
    scriptButton.setText("Select File Path");

    Text scriptLocation=new Text(scriptRow1, SWT.LEFT | SWT.BORDER | SWT.READ_ONLY);
    scriptLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    if(SMT.singleScriptFile)
    {
      scriptLocation.setText(SMT.dirSMT_LIBScript+File.separator+SMT.fileNameSMT_LIBScript);	 
    }
    else
    {
      scriptLocation.setText(SMT.dirSMT_LIBScript+File.separator+"*");	 
    }	 

    scriptButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        if(SMT.singleScriptFile)
        {
          FileDialog fileDialog = new FileDialog(shell, SWT.SAVE);
          fileDialog.setFilterPath(SMT.dirSMT_LIBScript);
          fileDialog.setFileName(SMT.fileNameSMT_LIBScript);
          fileDialog.open();
          String fileName = fileDialog.getFileName();     
          if (fileName == null) return;
          if (fileName.equals("")) return;
          if (fileName.equals("Untitled")) return;
          String fileDir = fileDialog.getFilterPath();
          if (fileDir == null) return;
          if (fileDir.equals("")) return;
          SMT.dirSMT_LIBScript=fileDir;
          SMT.fileNameSMT_LIBScript=fileName;
          scriptLocation.setText(fileDir+File.separator+fileName);
        }
        else
        {
          DirectoryDialog dirDialog = new DirectoryDialog(shell);
          dirDialog.setFilterPath(new File(SMT.dirSMT_LIBScript).getParent());
          String fileDir=dirDialog.open();
          if (fileDir == null) return;
          if (fileDir.equals("")) return;
          SMT.dirSMT_LIBScript=fileDir;
          scriptLocation.setText(fileDir+File.separator+"*");
        }		 		
      }
    });

    Composite scriptRow2 = new Composite(scriptGr, SWT.NONE);
    scriptRow2.setLayout(new GridLayout(2, false));
    scriptRow2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    Button writeToFileButton = new Button(scriptRow2, SWT.CHECK);
    writeToFileButton.setText("Write Script to File");
    writeToFileButton.setSelection(SMT.writeTranslatedSpecToFile);

    writeToFileButton.addSelectionListener(new SelectionAdapter() 
    {
      public void widgetSelected(SelectionEvent e) 
      {
        SMT.writeTranslatedSpecToFile=writeToFileButton.getSelection();
      }
    });

    Button scriptSingleFileButton = new Button(scriptRow2, SWT.CHECK);
    scriptSingleFileButton.setText("Use Single File");
    scriptSingleFileButton.setSelection(SMT.singleScriptFile);

    scriptSingleFileButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        SMT.singleScriptFile=scriptSingleFileButton.getSelection();
        if(scriptSingleFileButton.getSelection())
        {
          scriptLocation.setText(SMT.dirSMT_LIBScript+File.separator+SMT.fileNameSMT_LIBScript);
        }
        else
        {
          scriptLocation.setText(SMT.dirSMT_LIBScript+File.separator+"*");
        }
      }
    });

    Group additionalOptionsGr = new Group(shell, SWT.NONE);
    additionalOptionsGr.setText("Further Settings");
    additionalOptionsGr.setLayout(new GridLayout(2, false));
    additionalOptionsGr.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

    new Text(additionalOptionsGr, SWT.READ_ONLY).setText("Solver Timeout (sec)");
    Text tx1=new Text(additionalOptionsGr, SWT.LEFT | SWT.BORDER);
    tx1.setTextLimit(4);
    GridData gd=new GridData();
    gd.widthHint=40;
    tx1.setLayoutData(gd);
    //	 tx1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    tx1.setText(Long.toString(SMT.timeOut));

    tx1.addVerifyListener(new VerifyListener() {	
      @Override
      public void verifyText(VerifyEvent event) {
        if(!Character.isDigit(event.character)&&!Character.isISOControl(event.character))
          event.doit=false;			
      }
    });

    tx1.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent arg0) {
        long t;
        try
        {
          t=Long.parseLong(tx1.getText());
        }
        catch(NumberFormatException e)
        {
          tx1.setText(Long.toString(SMT.timeOut));
          return;
        }
        if(t>0)
        {
          SMT.timeOut=t;
        }							
      }
    });


    shell.setSize(900,500);
    shell.open();
  }
}
//---------------------------------------------------------------------------
// end of file
//---------------------------------------------------------------------------