// ---------------------------------------------------------------------------
// SymbolList.java
// A list of Unicode symbols for input in the editor area.
// $Id: SymbolList.java,v 1.4 2020/07/22 07:21:15 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.jku.at>
// Copyright (C) 2017-, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.jku.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package riscal.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

public class SymbolList
{
  /****************************************************************************
   * Construct window connected to denoted top Window
   * @param top the top window.
   * @return a window connected to the GUI.
   ***************************************************************************/
  public static SymbolList construct(TopWindow top)
  {
    return new SymbolList(top);
  }
  
  // the symbols and the associated key words
  private static String[] symbols = 
    {"ℤ", "ℕ", "≔",
     "⊤", "⊥", "¬", "∧", "∨", "⇒", "⇔", "∀", "∃",
     "∑", "∏", "≠", "≤", "≥", "⋅", "×",
     "∅", "∩", "∪", "⋂", "⋃", "∈", "∉", "⊆", "⊈",
     "⟨", "⟩"};
  private static String[] keywords = 
    {"Int", "Nat", ":=",
     "true", "false", "~", "/\\", "\\/", "=>", "<=>", "forall", "exists",
     "sum", "product", "~=", "<=", ">=", "*", "times",
     "{}", "intersect", "union", "Intersect", "Union", 
     "isin", "notin", "subseteq", "notsubseteq",
     "<<", ">>"};
  
  /****************************************************************************
   * Create symbol list using denoted top window.
   * @param top the top window.
   * @return a window connected to the GUI.
   ***************************************************************************/
  private SymbolList(TopWindow top)
  {
    Shell shell = new Shell(top.getDisplay(), SWT.CLOSE | SWT.BORDER);
    List list = new List (shell, SWT.MULTI | SWT.V_SCROLL);
    
    int n = symbols.length;
    for (int i = 0; i < n; i++)
    {
      list.add(symbols[i] + "\t(" + keywords[i] + ")");
    }
    list.deselectAll();
    list.addListener(SWT.Selection, e ->
    {
      int index = list.getSelectionIndex();
      if (index >= 0) top.insertText(symbols[index]); 
    }); 
    
    shell.setText("Symbols");
    shell.setLayout(new FillLayout());
    GC gc = new GC(list);
    int w = (int)(20*gc.getFontMetrics().getAverageCharacterWidth());
    int h = (int)(n*1.4*gc.getFontMetrics().getHeight());
    shell.setSize(w, h);
    shell.open();
  }
}
//---------------------------------------------------------------------------
//end of file
//---------------------------------------------------------------------------